@@ -1,4 +1,4 @@
-// $Id: tabledrag.js,v 1.13.2.1 2008/02/08 18:54:10 goba Exp $
+(function ($) {
 
 /**
  * Drag and drop table rows with field manipulation.
@@ -11,18 +11,14 @@
  * overriding the .onDrag, .onDrop, .row.onSwap, and .row.onIndent methods.
  * See blocks.js for an example of adding additional functionality to tableDrag.
  */
-Drupal.behaviors.tableDrag = function(context) {
-  for (var base in Drupal.settings.tableDrag) {
-    if (!$('#' + base + '.tabledrag-processed', context).size()) {
-      var tableSettings = Drupal.settings.tableDrag[base];
-
-      $('#' + base).filter(':not(.tabledrag-processed)').each(function() {
+Drupal.behaviors.tableDrag = {
+  attach: function (context, settings) {
+    for (var base in settings.tableDrag) {
+      $('#' + base, context).once('tabledrag', function () {
         // Create the new tableDrag instance. Save in the Drupal variable
         // to allow other scripts access to the object.
-        Drupal.tableDrag[base] = new Drupal.tableDrag(this, tableSettings);
+        Drupal.tableDrag[base] = new Drupal.tableDrag(this, settings.tableDrag[base]);
       });
-
-      $('#' + base).addClass('tabledrag-processed');
     }
   }
 };
@@ -35,7 +31,7 @@ Drupal.behaviors.tableDrag = function(context) {
  * @param tableSettings
  *   Settings for the table added via drupal_add_dragtable().
  */
-Drupal.tableDrag = function(table, tableSettings) {
+Drupal.tableDrag = function (table, tableSettings) {
   var self = this;
 
   // Required object variables.
@@ -59,13 +55,13 @@ Drupal.tableDrag = function(table, tableSettings) {
   // this table. For efficiency, large sections of code can be skipped if we
   // don't need to track horizontal movement and indentations.
   this.indentEnabled = false;
-  for (group in tableSettings) {
-    for (n in tableSettings[group]) {
-      if (tableSettings[group][n]['relationship'] == 'parent') {
+  for (var group in tableSettings) {
+    for (var n in tableSettings[group]) {
+      if (tableSettings[group][n].relationship == 'parent') {
         this.indentEnabled = true;
       }
-      if (tableSettings[group][n]['limit'] > 0) {
-        this.maxDepth = tableSettings[group][n]['limit'];
+      if (tableSettings[group][n].limit > 0) {
+        this.maxDepth = tableSettings[group][n].limit;
       }
     }
   }
@@ -73,79 +69,173 @@ Drupal.tableDrag = function(table, tableSettings) {
     this.indentCount = 1; // Total width of indents, set in makeDraggable.
     // Find the width of indentations to measure mouse movements against.
     // Because the table doesn't need to start with any indentations, we
-    // manually create an empty div, check it's width, then remove.
-    var indent = $(Drupal.theme('tableDragIndentation')).appendTo('body');
-    this.indentAmount = parseInt(indent.css('width'));
-    indent.remove();
+    // manually append 2 indentations in the first draggable row, measure
+    // the offset, then remove.
+    var indent = Drupal.theme('tableDragIndentation');
+    var testRow = $('<tr/>').addClass('draggable').appendTo(table);
+    var testCell = $('<td/>').appendTo(testRow).prepend(indent).prepend(indent);
+    this.indentAmount = $('.indentation', testCell).get(1).offsetLeft - $('.indentation', testCell).get(0).offsetLeft;
+    testRow.remove();
   }
 
   // Make each applicable row draggable.
-  $('tr.draggable', table).each(function() { self.makeDraggable(this); });
+  // Match immediate children of the parent element to allow nesting.
+  $('> tr.draggable, > tbody > tr.draggable', table).each(function () { self.makeDraggable(this); });
 
-  // Hide columns containing affected form elements.
-  this.hideColumns();
+  // Add a link before the table for users to show or hide weight columns.
+  $(table).before($('<a href="#" class="tabledrag-toggle-weight"></a>')
+    .attr('title', Drupal.t('Re-order rows by numerical weight instead of dragging.'))
+    .click(function () {
+      if ($.cookie('Drupal.tableDrag.showWeight') == 1) {
+        self.hideColumns();
+      }
+      else {
+        self.showColumns();
+      }
+      return false;
+    })
+    .wrap('<div class="tabledrag-toggle-weight-wrapper"></div>')
+    .parent()
+  );
+
+  // Initialize the specified columns (for example, weight or parent columns)
+  // to show or hide according to user preference. This aids accessibility
+  // so that, e.g., screen reader users can choose to enter weight values and
+  // manipulate form elements directly, rather than using drag-and-drop..
+  self.initColumns();
 
   // Add mouse bindings to the document. The self variable is passed along
   // as event handlers do not have direct access to the tableDrag object.
-  $(document).bind('mousemove', function(event) { self.dragRow(event, self); return false; });
-  $(document).bind('mouseup', function(event) { self.dropRow(event, self); });
+  $(document).bind('mousemove', function (event) { return self.dragRow(event, self); });
+  $(document).bind('mouseup', function (event) { return self.dropRow(event, self); });
 };
 
 /**
- * Hide the columns containing form elements according to the settings for
- * this tableDrag instance.
+ * Initialize columns containing form elements to be hidden by default,
+ * according to the settings for this tableDrag instance.
+ *
+ * Identify and mark each cell with a CSS class so we can easily toggle
+ * show/hide it. Finally, hide columns if user does not have a
+ * 'Drupal.tableDrag.showWeight' cookie.
  */
-Drupal.tableDrag.prototype.hideColumns = function() {
+Drupal.tableDrag.prototype.initColumns = function () {
   for (var group in this.tableSettings) {
     // Find the first field in this group.
     for (var d in this.tableSettings[group]) {
-      if ($('.' + this.tableSettings[group][d]['target'], this.table).size()) {
-        var hidden = this.tableSettings[group][d]['hidden'];
-        var field = $('.' + this.tableSettings[group][d]['target'] + ':first', this.table);
-        var cell = field.parents('td:first, th:first');
+      var field = $('.' + this.tableSettings[group][d].target + ':first', this.table);
+      if (field.size() && this.tableSettings[group][d].hidden) {
+        var hidden = this.tableSettings[group][d].hidden;
+        var cell = field.parents('td:first');
         break;
       }
     }
-    // Hide the column containing this field.
-    if (hidden && cell[0] && cell.css('display') != 'none') {
+
+    // Mark the column containing this field so it can be hidden.
+    if (hidden && cell[0]) {
       // Add 1 to our indexes. The nth-child selector is 1 based, not 0 based.
-      var columnIndex = $('td', field.parents('tr:first')).index(cell.get(0)) + 1;
-      var headerIndex = $('td:not(:hidden)', field.parents('tr:first')).index(cell.get(0)) + 1;
-      $('tbody tr', this.table).each(function() {
-        // Find and hide the cell in the table body.
-        var cell = $('td:nth-child('+ columnIndex +')', this);
-        if (cell.size()) {
-          cell.css('display', 'none');
-        }
-        // We might be dealing with a row spanning the entire table.
-        // Reduce the colspan on the first cell to prevent the cell from
-        // overshooting the table.
-        else {
-          cell = $('td:first', this);
-          cell.attr('colspan', cell.attr('colspan') - 1);
-        }
-      });
-      $('thead tr', this.table).each(function() {
-        // Remove table header cells entirely (Safari doesn't hide properly).
-        var th = $('th:nth-child('+ headerIndex +')', this);
-        if (th.size()) {
-          th.remove();
+      // Match immediate children of the parent element to allow nesting.
+      var columnIndex = $('> td', cell.parent()).index(cell.get(0)) + 1;
+      $('> thead > tr, > tbody > tr, > tr', this.table).each(function () {
+        // Get the columnIndex and adjust for any colspans in this row.
+        var index = columnIndex;
+        var cells = $(this).children();
+        cells.each(function (n) {
+          if (n < index && this.colSpan && this.colSpan > 1) {
+            index -= this.colSpan - 1;
+          }
+        });
+        if (index > 0) {
+          cell = cells.filter(':nth-child(' + index + ')');
+          if (cell[0].colSpan && cell[0].colSpan > 1) {
+            // If this cell has a colspan, mark it so we can reduce the colspan.
+            cell.addClass('tabledrag-has-colspan');
+          }
+          else {
+            // Mark this cell so we can hide it.
+            cell.addClass('tabledrag-hide');
+          }
         }
       });
     }
   }
+
+  // Now hide cells and reduce colspans unless cookie indicates previous choice.
+  // Set a cookie if it is not already present.
+  if ($.cookie('Drupal.tableDrag.showWeight') === null) {
+    $.cookie('Drupal.tableDrag.showWeight', 0, {
+      path: Drupal.settings.basePath,
+      // The cookie expires in one year.
+      expires: 365
+    });
+    this.hideColumns();
+  }
+  // Check cookie value and show/hide weight columns accordingly.
+  else {
+    if ($.cookie('Drupal.tableDrag.showWeight') == 1) {
+      this.showColumns();
+    }
+    else {
+      this.hideColumns();
+    }
+  }
+};
+
+/**
+ * Hide the columns containing weight/parent form elements.
+ * Undo showColumns().
+ */
+Drupal.tableDrag.prototype.hideColumns = function () {
+  // Hide weight/parent cells and headers.
+  $('.tabledrag-hide', 'table.tabledrag-processed').css('display', 'none');
+  // Show TableDrag handles.
+  $('.tabledrag-handle', 'table.tabledrag-processed').css('display', '');
+  // Reduce the colspan of any effected multi-span columns.
+  $('.tabledrag-has-colspan', 'table.tabledrag-processed').each(function () {
+    this.colSpan = this.colSpan - 1;
+  });
+  // Change link text.
+  $('.tabledrag-toggle-weight').text(Drupal.t('Show row weights'));
+  // Change cookie.
+  $.cookie('Drupal.tableDrag.showWeight', 0, {
+    path: Drupal.settings.basePath,
+    // The cookie expires in one year.
+    expires: 365
+  });
+};
+
+/**
+ * Show the columns containing weight/parent form elements
+ * Undo hideColumns().
+ */
+Drupal.tableDrag.prototype.showColumns = function () {
+  // Show weight/parent cells and headers.
+  $('.tabledrag-hide', 'table.tabledrag-processed').css('display', '');
+  // Hide TableDrag handles.
+  $('.tabledrag-handle', 'table.tabledrag-processed').css('display', 'none');
+  // Increase the colspan for any columns where it was previously reduced.
+  $('.tabledrag-has-colspan', 'table.tabledrag-processed').each(function () {
+    this.colSpan = this.colSpan + 1;
+  });
+  // Change link text.
+  $('.tabledrag-toggle-weight').text(Drupal.t('Hide row weights'));
+  // Change cookie.
+  $.cookie('Drupal.tableDrag.showWeight', 1, {
+    path: Drupal.settings.basePath,
+    // The cookie expires in one year.
+    expires: 365
+  });
 };
 
 /**
  * Find the target used within a particular row and group.
  */
-Drupal.tableDrag.prototype.rowSettings = function(group, row) {
+Drupal.tableDrag.prototype.rowSettings = function (group, row) {
   var field = $('.' + group, row);
-  for (delta in this.tableSettings[group]) {
-    var targetClass = this.tableSettings[group][delta]['target'];
+  for (var delta in this.tableSettings[group]) {
+    var targetClass = this.tableSettings[group][delta].target;
     if (field.is('.' + targetClass)) {
       // Return a copy of the row settings.
-      var rowSettings = new Object();
+      var rowSettings = {};
       for (var n in this.tableSettings[group][delta]) {
         rowSettings[n] = this.tableSettings[group][delta][n];
       }
@@ -157,13 +247,14 @@ Drupal.tableDrag.prototype.rowSettings = function(group, row) {
 /**
  * Take an item and add event handlers to make it become draggable.
  */
-Drupal.tableDrag.prototype.makeDraggable = function(item) {
+Drupal.tableDrag.prototype.makeDraggable = function (item) {
   var self = this;
 
   // Create the handle.
   var handle = $('<a href="#" class="tabledrag-handle"><div class="handle">&nbsp;</div></a>').attr('title', Drupal.t('Drag to re-order'));
   // Insert the handle after indentations (if any).
-  if ($('td:first .indentation:last', item).after(handle).size()) {
+  if ($('td:first .indentation:last', item).length) {
+    $('td:first .indentation:last', item).after(handle);
     // Update the total width of indentation in this entire table.
     self.indentCount = Math.max($('.indentation', item).size(), self.indentCount);
   }
@@ -172,16 +263,16 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
   }
 
   // Add hover action for the handle.
-  handle.hover(function() {
+  handle.hover(function () {
     self.dragObject == null ? $(this).addClass('tabledrag-handle-hover') : null;
-  }, function() {
+  }, function () {
     self.dragObject == null ? $(this).removeClass('tabledrag-handle-hover') : null;
   });
 
   // Add the mousedown action for the handle.
-  handle.mousedown(function(event) {
+  handle.mousedown(function (event) {
     // Create a new dragObject recording the event information.
-    self.dragObject = new Object();
+    self.dragObject = {};
     self.dragObject.initMouseOffset = self.getMouseOffset(item, event);
     self.dragObject.initMouseCoords = self.mouseCoords(event);
     if (self.indentEnabled) {
@@ -197,7 +288,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
     self.rowObject = new self.row(item, 'mouse', self.indentEnabled, self.maxDepth, true);
 
     // Save the position of the table.
-    self.table.topY = self.getPosition(self.table).y;
+    self.table.topY = $(self.table).offset().top;
     self.table.bottomY = self.table.topY + self.table.offsetHeight;
 
     // Add classes to the handle and row.
@@ -225,18 +316,18 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
   });
 
   // Prevent the anchor tag from jumping us to the top of the page.
-  handle.click(function() {
+  handle.click(function () {
     return false;
   });
 
   // Similar to the hover event, add a class when the handle is focused.
-  handle.focus(function() {
+  handle.focus(function () {
     $(this).addClass('tabledrag-handle-hover');
     self.safeBlur = true;
   });
 
   // Remove the handle class on blur and fire the same function as a mouseup.
-  handle.blur(function(event) {
+  handle.blur(function (event) {
     $(this).removeClass('tabledrag-handle-hover');
     if (self.rowObject && self.safeBlur) {
       self.dropRow(event, self);
@@ -244,7 +335,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
   });
 
   // Add arrow-key support to the handle.
-  handle.keydown(function(event) {
+  handle.keydown(function (event) {
     // If a rowObject doesn't yet exist and this isn't the tab key.
     if (event.keyCode != 9 && !self.rowObject) {
       self.rowObject = new self.row(item, 'keyboard', self.indentEnabled, self.maxDepth, true);
@@ -269,7 +360,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
           keyChange = true;
 
           if ($(item).is('.tabledrag-root')) {
-            // Swap with the previous top-level row..
+            // Swap with the previous top-level row.
             var groupHeight = 0;
             while (previousRow && $('.indentation', previousRow).size()) {
               previousRow = $(previousRow).prev('tr').get(0);
@@ -287,7 +378,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
             self.rowObject.swap('before', previousRow);
             self.rowObject.interval = null;
             self.rowObject.indent(0);
-            window.scrollBy(0, -parseInt(item.offsetHeight));
+            window.scrollBy(0, -parseInt(item.offsetHeight, 10));
           }
           handle.get(0).focus(); // Regain focus after the DOM manipulation.
         }
@@ -313,11 +404,13 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
             var groupHeight = 0;
             nextGroup = new self.row(nextRow, 'keyboard', self.indentEnabled, self.maxDepth, false);
             if (nextGroup) {
-              $(nextGroup.group).each(function () {groupHeight += $(this).is(':hidden') ? 0 : this.offsetHeight});
+              $(nextGroup.group).each(function () {
+                groupHeight += $(this).is(':hidden') ? 0 : this.offsetHeight;
+              });
               nextGroupRow = $(nextGroup.group).filter(':last').get(0);
               self.rowObject.swap('after', nextGroupRow);
               // No need to check for indentation, 0 is the only valid one.
-              window.scrollBy(0, parseInt(groupHeight));
+              window.scrollBy(0, parseInt(groupHeight, 10));
             }
           }
           else {
@@ -325,7 +418,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
             self.rowObject.swap('after', nextRow);
             self.rowObject.interval = null;
             self.rowObject.indent(0);
-            window.scrollBy(0, parseInt(item.offsetHeight));
+            window.scrollBy(0, parseInt(item.offsetHeight, 10));
           }
           handle.get(0).focus(); // Regain focus after the DOM manipulation.
         }
@@ -349,9 +442,9 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
   });
 
   // Compatibility addition, return false on keypress to prevent unwanted scrolling.
-  // IE and Safari will supress scrolling on keydown, but all other browsers
+  // IE and Safari will suppress scrolling on keydown, but all other browsers
   // need to return false on keypress. http://www.quirksmode.org/js/keys.html
-  handle.keypress(function(event) {
+  handle.keypress(function (event) {
     switch (event.keyCode) {
       case 37: // Left arrow.
       case 38: // Up arrow.
@@ -365,7 +458,7 @@ Drupal.tableDrag.prototype.makeDraggable = function(item) {
 /**
  * Mousemove event handler, bound to document.
  */
-Drupal.tableDrag.prototype.dragRow = function(event, self) {
+Drupal.tableDrag.prototype.dragRow = function (event, self) {
   if (self.dragObject) {
     self.currentMouseCoords = self.mouseCoords(event);
 
@@ -403,7 +496,7 @@ Drupal.tableDrag.prototype.dragRow = function(event, self) {
     if (self.indentEnabled) {
       var xDiff = self.currentMouseCoords.x - self.dragObject.indentMousePos.x;
       // Set the number of indentations the mouse has been moved left or right.
-      var indentDiff = parseInt(xDiff / self.indentAmount * self.rtl);
+      var indentDiff = Math.round(xDiff / self.indentAmount * self.rtl);
       // Indent the row with our estimated diff, which may be further
       // restricted according to the rows around this row.
       var indentChange = self.rowObject.indent(indentDiff);
@@ -411,6 +504,8 @@ Drupal.tableDrag.prototype.dragRow = function(event, self) {
       self.dragObject.indentMousePos.x += self.indentAmount * indentChange * self.rtl;
       self.indentCount = Math.max(self.indentCount, self.rowObject.indents);
     }
+
+    return false;
   }
 };
 
@@ -418,7 +513,7 @@ Drupal.tableDrag.prototype.dragRow = function(event, self) {
  * Mouseup event handler, bound to document.
  * Blur event handler, bound to drag handle for keyboard support.
  */
-Drupal.tableDrag.prototype.dropRow = function(event, self) {
+Drupal.tableDrag.prototype.dropRow = function (event, self) {
   // Drop row functionality shared between mouseup and blur events.
   if (self.rowObject != null) {
     var droppedRow = self.rowObject.element;
@@ -432,7 +527,7 @@ Drupal.tableDrag.prototype.dropRow = function(event, self) {
       for (var group in self.tableSettings) {
         var rowSettings = self.rowSettings(group, droppedRow);
         if (rowSettings.relationship == 'group') {
-          for (n in self.rowObject.children) {
+          for (var n in self.rowObject.children) {
             self.updateField(self.rowObject.children[n], group);
           }
         }
@@ -440,7 +535,7 @@ Drupal.tableDrag.prototype.dropRow = function(event, self) {
 
       self.rowObject.markChanged();
       if (self.changed == false) {
-        $(Drupal.theme('tableDragChangedWarning')).insertAfter(self.table).hide().fadeIn('slow');
+        $(Drupal.theme('tableDragChangedWarning')).insertBefore(self.table).hide().fadeIn('slow');
         self.changed = true;
       }
     }
@@ -472,41 +567,16 @@ Drupal.tableDrag.prototype.dropRow = function(event, self) {
   }
 };
 
-/**
- * Get the position of an element by adding up parent offsets in the DOM tree.
- */
-Drupal.tableDrag.prototype.getPosition = function(element){
-  var left = 0;
-  var top  = 0;
-  // Because Safari doesn't report offsetHeight on table rows, but does on table
-  // cells, grab the firstChild of the row and use that instead.
-  // http://jacob.peargrove.com/blog/2006/technical/table-row-offsettop-bug-in-safari
-  if (element.offsetHeight == 0) {
-    element = element.firstChild; // a table cell
-  }
-
-  while (element.offsetParent){
-    left   += element.offsetLeft;
-    top    += element.offsetTop;
-    element = element.offsetParent;
-  }
-
-  left += element.offsetLeft;
-  top  += element.offsetTop;
-
-  return {x:left, y:top};
-};
-
 /**
  * Get the mouse coordinates from the event (allowing for browser differences).
  */
-Drupal.tableDrag.prototype.mouseCoords = function(event){
+Drupal.tableDrag.prototype.mouseCoords = function (event) {
   if (event.pageX || event.pageY) {
-    return {x:event.pageX, y:event.pageY};
+    return { x: event.pageX, y: event.pageY };
   }
   return {
-    x:event.clientX + document.body.scrollLeft - document.body.clientLeft,
-    y:event.clientY + document.body.scrollTop  - document.body.clientTop
+    x: event.clientX + document.body.scrollLeft - document.body.clientLeft,
+    y: event.clientY + document.body.scrollTop  - document.body.clientTop
   };
 };
 
@@ -514,10 +584,10 @@ Drupal.tableDrag.prototype.mouseCoords = function(event){
  * Given a target element and a mouse event, get the mouse offset from that
  * element. To do this we need the element's position and the mouse position.
  */
-Drupal.tableDrag.prototype.getMouseOffset = function(target, event) {
-  var docPos   = this.getPosition(target);
+Drupal.tableDrag.prototype.getMouseOffset = function (target, event) {
+  var docPos   = $(target).offset();
   var mousePos = this.mouseCoords(event);
-  return {x:mousePos.x - docPos.x, y:mousePos.y - docPos.y};
+  return { x: mousePos.x - docPos.left, y: mousePos.y - docPos.top };
 };
 
 /**
@@ -529,32 +599,40 @@ Drupal.tableDrag.prototype.getMouseOffset = function(target, event) {
  * @param y
  *   The y coordinate of the mouse on the page (not the screen).
  */
-Drupal.tableDrag.prototype.findDropTargetRow = function(x, y) {
-  var rows = this.table.tBodies[0].rows;
-  for (var n=0; n<rows.length; n++) {
+Drupal.tableDrag.prototype.findDropTargetRow = function (x, y) {
+  var rows = $(this.table.tBodies[0].rows).not(':hidden');
+  for (var n = 0; n < rows.length; n++) {
     var row = rows[n];
     var indentDiff = 0;
-    // Safari fix see Drupal.tableDrag.prototype.getPosition()
+    var rowY = $(row).offset().top;
+    // Because Safari does not report offsetHeight on table rows, but does on
+    // table cells, grab the firstChild of the row and use that instead.
+    // http://jacob.peargrove.com/blog/2006/technical/table-row-offsettop-bug-in-safari.
     if (row.offsetHeight == 0) {
-      var rowY = this.getPosition(row.firstChild).y;
-      var rowHeight = parseInt(row.firstChild.offsetHeight)/2;
+      var rowHeight = parseInt(row.firstChild.offsetHeight, 10) / 2;
     }
     // Other browsers.
     else {
-      var rowY = this.getPosition(row).y;
-      var rowHeight = parseInt(row.offsetHeight)/2;
+      var rowHeight = parseInt(row.offsetHeight, 10) / 2;
     }
 
     // Because we always insert before, we need to offset the height a bit.
     if ((y > (rowY - rowHeight)) && (y < (rowY + rowHeight))) {
       if (this.indentEnabled) {
         // Check that this row is not a child of the row being dragged.
-        for (n in this.rowObject.group) {
+        for (var n in this.rowObject.group) {
           if (this.rowObject.group[n] == row) {
             return null;
           }
         }
       }
+      else {
+        // Do not allow a row to be swapped with itself.
+        if (row == this.rowObject.element) {
+          return null;
+        }
+      }
+
       // Check that swapping with this row is allowed.
       if (!this.rowObject.isValidSwap(row)) {
         return null;
@@ -579,10 +657,10 @@ Drupal.tableDrag.prototype.findDropTargetRow = function(x, y) {
  * @param changedRow
  *   DOM object for the row that was just dropped.
  */
-Drupal.tableDrag.prototype.updateFields = function(changedRow) {
+Drupal.tableDrag.prototype.updateFields = function (changedRow) {
   for (var group in this.tableSettings) {
     // Each group may have a different setting for relationship, so we find
-    // the source rows for each seperately.
+    // the source rows for each separately.
     this.updateField(changedRow, group);
   }
 };
@@ -596,10 +674,10 @@ Drupal.tableDrag.prototype.updateFields = function(changedRow) {
  * @param group
  *   The settings group on which field updates will occur.
  */
-Drupal.tableDrag.prototype.updateField = function(changedRow, group) {
+Drupal.tableDrag.prototype.updateField = function (changedRow, group) {
   var rowSettings = this.rowSettings(group, changedRow);
 
-  // Set the row as it's own target.
+  // Set the row as its own target.
   if (rowSettings.relationship == 'self' || rowSettings.relationship == 'group') {
     var sourceRow = changedRow;
   }
@@ -643,10 +721,10 @@ Drupal.tableDrag.prototype.updateField = function(changedRow, group) {
     // Otherwise we went all the way to the left of the table without finding
     // a parent, meaning this item has been placed at the root level.
     else {
-      // Use the first row in the table as source, because it's garanteed to
+      // Use the first row in the table as source, because it's guaranteed to
       // be at the root level. Find the first item, then compare this row
       // against it as a sibling.
-      sourceRow = $('tr.draggable:first').get(0);
+      sourceRow = $(this.table).find('tr.draggable:first').get(0);
       if (sourceRow == this.rowObject.element) {
         sourceRow = $(this.rowObject.group[this.rowObject.group.length - 1]).next('tr.draggable').get(0);
       }
@@ -686,14 +764,14 @@ Drupal.tableDrag.prototype.updateField = function(changedRow, group) {
         var siblings = this.rowObject.findSiblings(rowSettings);
         if ($(targetElement).is('select')) {
           // Get a list of acceptable values.
-          var values = new Array();
-          $('option', targetElement).each(function() {
+          var values = [];
+          $('option', targetElement).each(function () {
             values.push(this.value);
           });
           var maxVal = values[values.length - 1];
           // Populate the values in the siblings.
-          $(targetClass, siblings).each(function() {
-            // If there are more items than possible values, assign the maximum value to the row. 
+          $(targetClass, siblings).each(function () {
+            // If there are more items than possible values, assign the maximum value to the row.
             if (values.length > 0) {
               this.value = values.shift();
             }
@@ -704,8 +782,8 @@ Drupal.tableDrag.prototype.updateField = function(changedRow, group) {
         }
         else {
           // Assume a numeric input field.
-          var weight = parseInt($(targetClass, siblings[0]).val()) || 0;
-          $(targetClass, siblings).each(function() {
+          var weight = parseInt($(targetClass, siblings[0]).val(), 10) || 0;
+          $(targetClass, siblings).each(function () {
             this.value = weight;
             weight++;
           });
@@ -720,7 +798,7 @@ Drupal.tableDrag.prototype.updateField = function(changedRow, group) {
  * different one, removing any special classes that the destination row
  * may have had.
  */
-Drupal.tableDrag.prototype.copyDragClasses = function(sourceRow, targetRow, group) {
+Drupal.tableDrag.prototype.copyDragClasses = function (sourceRow, targetRow, group) {
   var sourceElement = $('.' + group, sourceRow);
   var targetElement = $('.' + group, targetRow);
   if (sourceElement.length && targetElement.length) {
@@ -728,7 +806,7 @@ Drupal.tableDrag.prototype.copyDragClasses = function(sourceRow, targetRow, grou
   }
 };
 
-Drupal.tableDrag.prototype.checkScroll = function(cursorY) {
+Drupal.tableDrag.prototype.checkScroll = function (cursorY) {
   var de  = document.documentElement;
   var b  = document.body;
 
@@ -750,10 +828,10 @@ Drupal.tableDrag.prototype.checkScroll = function(cursorY) {
   }
 };
 
-Drupal.tableDrag.prototype.setScroll = function(scrollAmount) {
+Drupal.tableDrag.prototype.setScroll = function (scrollAmount) {
   var self = this;
 
-  this.scrollInterval = setInterval(function() {
+  this.scrollInterval = setInterval(function () {
     // Update the scroll values stored in the object.
     self.checkScroll(self.currentMouseCoords.y);
     var aboveTable = self.scrollY > self.table.topY;
@@ -764,28 +842,27 @@ Drupal.tableDrag.prototype.setScroll = function(scrollAmount) {
   }, this.scrollSettings.interval);
 };
 
-Drupal.tableDrag.prototype.restripeTable = function() {
-  // :even and :odd are reversed because jquery counts from 0 and
+Drupal.tableDrag.prototype.restripeTable = function () {
+  // :even and :odd are reversed because jQuery counts from 0 and
   // we count from 1, so we're out of sync.
-  $('tr.draggable', this.table)
-    .filter(':odd').filter('.odd')
-      .removeClass('odd').addClass('even')
-    .end().end()
-    .filter(':even').filter('.even')
-      .removeClass('even').addClass('odd');
+  // Match immediate children of the parent element to allow nesting.
+  $('> tbody > tr.draggable:visible, > tr.draggable:visible', this.table)
+    .removeClass('odd even')
+    .filter(':odd').addClass('even').end()
+    .filter(':even').addClass('odd');
 };
 
 /**
  * Stub function. Allows a custom handler when a row begins dragging.
  */
-Drupal.tableDrag.prototype.onDrag = function() {
+Drupal.tableDrag.prototype.onDrag = function () {
   return null;
 };
 
 /**
  * Stub function. Allows a custom handler when a row is dropped.
  */
-Drupal.tableDrag.prototype.onDrop = function() {
+Drupal.tableDrag.prototype.onDrop = function () {
   return null;
 };
 
@@ -803,10 +880,10 @@ Drupal.tableDrag.prototype.onDrop = function() {
  * @param addClasses
  *   Whether we want to add classes to this row to indicate child relationships.
  */
-Drupal.tableDrag.prototype.row = function(tableRow, method, indentEnabled, maxDepth, addClasses) {
+Drupal.tableDrag.prototype.row = function (tableRow, method, indentEnabled, maxDepth, addClasses) {
   this.element = tableRow;
   this.method = method;
-  this.group = new Array(tableRow);
+  this.group = [tableRow];
   this.groupDepth = $('.indentation', tableRow).size();
   this.changed = false;
   this.table = $(tableRow).parents('table:first').get(0);
@@ -831,10 +908,10 @@ Drupal.tableDrag.prototype.row = function(tableRow, method, indentEnabled, maxDe
  * @param addClasses
  *   Whether we want to add classes to this row to indicate child relationships.
  */
-Drupal.tableDrag.prototype.row.prototype.findChildren = function(addClasses) {
+Drupal.tableDrag.prototype.row.prototype.findChildren = function (addClasses) {
   var parentIndentation = this.indents;
   var currentRow = $(this.element, this.table).next('tr.draggable');
-  var rows = new Array();
+  var rows = [];
   var child = 0;
   while (currentRow.length) {
     var rowIndentation = $('.indentation', currentRow).length;
@@ -843,7 +920,7 @@ Drupal.tableDrag.prototype.row.prototype.findChildren = function(addClasses) {
       child++;
       rows.push(currentRow[0]);
       if (addClasses) {
-        $('.indentation', currentRow).each(function(indentNum) {
+        $('.indentation', currentRow).each(function (indentNum) {
           if (child == 1 && (indentNum == parentIndentation)) {
             $(this).addClass('tree-child-first');
           }
@@ -873,7 +950,7 @@ Drupal.tableDrag.prototype.row.prototype.findChildren = function(addClasses) {
  * @param row
  *   DOM object for the row being considered for swapping.
  */
-Drupal.tableDrag.prototype.row.prototype.isValidSwap = function(row) {
+Drupal.tableDrag.prototype.row.prototype.isValidSwap = function (row) {
   if (this.indentEnabled) {
     var prevRow, nextRow;
     if (this.direction == 'down') {
@@ -908,8 +985,10 @@ Drupal.tableDrag.prototype.row.prototype.isValidSwap = function(row) {
  * @param row
  *   DOM element what will be swapped with the row group.
  */
-Drupal.tableDrag.prototype.row.prototype.swap = function(position, row) {
+Drupal.tableDrag.prototype.row.prototype.swap = function (position, row) {
+  Drupal.detachBehaviors(this.group, Drupal.settings, 'move');
   $(row)[position](this.group);
+  Drupal.attachBehaviors(this.group, Drupal.settings);
   this.changed = true;
   this.onSwap(row);
 };
@@ -933,8 +1012,11 @@ Drupal.tableDrag.prototype.row.prototype.validIndentInterval = function (prevRow
   minIndent = nextRow ? $('.indentation', nextRow).size() : 0;
 
   // Maximum indentation:
-  if (!prevRow || $(this.element).is('.tabledrag-root')) {
-    // Do not indent the first row in the table or 'root' rows..
+  if (!prevRow || $(prevRow).is(':not(.draggable)') || $(this.element).is('.tabledrag-root')) {
+    // Do not indent:
+    // - the first row in the table,
+    // - rows dragged below a non-draggable row,
+    // - 'root' rows.
     maxIndent = 0;
   }
   else {
@@ -946,8 +1028,8 @@ Drupal.tableDrag.prototype.row.prototype.validIndentInterval = function (prevRow
     }
   }
 
-  return {'min':minIndent, 'max':maxIndent};
-}
+  return { 'min': minIndent, 'max': maxIndent };
+};
 
 /**
  * Indent a row within the legal bounds of the table.
@@ -957,7 +1039,7 @@ Drupal.tableDrag.prototype.row.prototype.validIndentInterval = function (prevRow
  *   positive or negative). This number will be adjusted to nearest valid
  *   indentation level for the row.
  */
-Drupal.tableDrag.prototype.row.prototype.indent = function(indentDiff) {
+Drupal.tableDrag.prototype.row.prototype.indent = function (indentDiff) {
   // Determine the valid indentations interval if not available yet.
   if (!this.interval) {
     prevRow = $(this.element).prev('tr').get(0);
@@ -994,24 +1076,24 @@ Drupal.tableDrag.prototype.row.prototype.indent = function(indentDiff) {
 
 /**
  * Find all siblings for a row, either according to its subgroup or indentation.
- * Note that the passed in row is included in the list of siblings.
+ * Note that the passed-in row is included in the list of siblings.
  *
  * @param settings
  *   The field settings we're using to identify what constitutes a sibling.
  */
-Drupal.tableDrag.prototype.row.prototype.findSiblings = function(rowSettings) {
-  var siblings = new Array();
-  var directions = new Array('prev', 'next');
+Drupal.tableDrag.prototype.row.prototype.findSiblings = function (rowSettings) {
+  var siblings = [];
+  var directions = ['prev', 'next'];
   var rowIndentation = this.indents;
-  for (var d in directions) {
+  for (var d = 0; d < directions.length; d++) {
     var checkRow = $(this.element)[directions[d]]();
     while (checkRow.length) {
       // Check that the sibling contains a similar target field.
       if ($('.' + rowSettings.target, checkRow)) {
         // Either add immediately if this is a flat table, or check to ensure
-        // that this row has the same level of indentaiton.
+        // that this row has the same level of indentation.
         if (this.indentEnabled) {
-          var checkRowIndentation = $('.indentation', checkRow).length
+          var checkRowIndentation = $('.indentation', checkRow).length;
         }
 
         if (!(this.indentEnabled) || (checkRowIndentation == rowIndentation)) {
@@ -1040,8 +1122,8 @@ Drupal.tableDrag.prototype.row.prototype.findSiblings = function(rowSettings) {
 /**
  * Remove indentation helper classes from the current row group.
  */
-Drupal.tableDrag.prototype.row.prototype.removeIndentClasses = function() {
-  for (n in this.children) {
+Drupal.tableDrag.prototype.row.prototype.removeIndentClasses = function () {
+  for (var n in this.children) {
     $('.indentation', this.children[n])
       .removeClass('tree-child')
       .removeClass('tree-child-first')
@@ -1053,7 +1135,7 @@ Drupal.tableDrag.prototype.row.prototype.removeIndentClasses = function() {
 /**
  * Add an asterisk or other marker to the changed row.
  */
-Drupal.tableDrag.prototype.row.prototype.markChanged = function() {
+Drupal.tableDrag.prototype.row.prototype.markChanged = function () {
   var marker = Drupal.theme('tableDragChangedMarker');
   var cell = $('td:first', this.element);
   if ($('span.tabledrag-changed', cell).length == 0) {
@@ -1064,14 +1146,14 @@ Drupal.tableDrag.prototype.row.prototype.markChanged = function() {
 /**
  * Stub function. Allows a custom handler when a row is indented.
  */
-Drupal.tableDrag.prototype.row.prototype.onIndent = function() {
+Drupal.tableDrag.prototype.row.prototype.onIndent = function () {
   return null;
 };
 
 /**
  * Stub function. Allows a custom handler when a row is swapped.
  */
-Drupal.tableDrag.prototype.row.prototype.onSwap = function(swappedRow) {
+Drupal.tableDrag.prototype.row.prototype.onSwap = function (swappedRow) {
   return null;
 };
 
@@ -1084,5 +1166,7 @@ Drupal.theme.prototype.tableDragIndentation = function () {
 };
 
 Drupal.theme.prototype.tableDragChangedWarning = function () {
-  return '<div class="warning">' + Drupal.theme('tableDragChangedMarker') + ' ' + Drupal.t("Changes made in this table will not be saved until the form is submitted.") + '</div>';
+  return '<div class="tabledrag-changed-warning messages warning">' + Drupal.theme('tableDragChangedMarker') + ' ' + Drupal.t('Changes made in this table will not be saved until the form is submitted.') + '</div>';
 };
+
+})(jQuery);
